@@ -121,6 +121,11 @@ class Arrays:
             return self.metadata['bkg_type']
         # TODO: Something based on filename
 
+    @property
+    def data_type(self):
+        if 'data_type' in self.metadata:
+            return self.metadata['data_type']
+
     @property
     def triggers(self):
         return triggers_per_year[self.year]
@@ -152,11 +157,13 @@ def local_copy(remote):
                 pass
 
 
-def open_root(rootfile):
+def open_root(rootfile, load_gen=True):
     """
     Returns an Arrays object from a rootfile (unfiltered).
     """
     branches = [
+        'Jets.fCoordinates.fPt', 'Jets.fCoordinates.fEta',
+        'Jets.fCoordinates.fPhi',
         'JetsAK8.fCoordinates.fPt',
         'JetsAK15.fCoordinates.fPt', 'JetsAK15.fCoordinates.fEta',
         'JetsAK15.fCoordinates.fPhi', 'JetsAK15.fCoordinates.fE',
@@ -173,15 +180,22 @@ def open_root(rootfile):
         'HBHENoiseFilter', 'HBHEIsoNoiseFilter', 'eeBadScFilter',
         'ecalBadCalibFilter' if UL else 'ecalBadCalibReducedFilter',
         'BadPFMuonFilter', 'BadChargedCandidateFilter', 'globalSuperTightHalo2016Filter',
-        'Weight',
-        'madHT', 'GenMET', 
+        'EcalDeadCellBoundaryEnergyFilter',
+        'EcalDeadCellTriggerPrimitiveFilter'
+        ]
+
+    if load_gen:
+        # Only available for simulation, not data
+        branches.extend([
+        'Weight', 
+        'madHT', 'GenMET',
         'GenParticles_PdgId',
         'GenParticles_Status',
         'GenParticles.fCoordinates.fPt',
         'GenParticles.fCoordinates.fEta',
         'GenParticles.fCoordinates.fPhi',
         'GenParticles.fCoordinates.fE',
-        ]
+        ])
 
     with local_copy(rootfile) as local:
         tree = uproot.open(local + ':TreeMaker2/PreSelection')
@@ -194,6 +208,13 @@ def open_root(rootfile):
     return arrays
 
 
+def open_data_root(rootfile):
+    """
+    Like open_root but for data
+    """
+    return open_root(rootfile, load_gen=False)
+
+
 def calc_dphi(phi1, phi2):
     """
     Calculates delta phi. Assures output is within -pi .. pi.
@@ -226,6 +247,73 @@ def calculate_mt(pt, eta, phi, e, met, metphi):
     return mt
 
 
+def cr_filter_preselection(array):
+    """
+    Preselection for the control region.
+    
+    w.r.t. `filter_preselection`, does not include:
+    - leading ak8 jet pt > 500
+    - rtx > 1.1
+
+    but includes:
+    - at least 2 ak4 jets
+    - both ak4 jets < 2.4 eta
+    """
+    copy = array.copy()
+    a = copy.array
+    cutflow = copy.cutflow
+
+    # AK8Jet.pT>500
+    a = a[ak.count(a['JetsAK8.fCoordinates.fPt'], axis=-1)>=1] # At least one jet
+    cutflow['ak8_njets>0'] = len(a)
+
+    # Triggers
+    trigger_indices = np.array([copy.trigger_branch.index(t) for t in copy.triggers])
+    if len(a):
+        trigger_decisions = a['TriggerPass'].to_numpy()[:,trigger_indices]
+        a = a[(trigger_decisions == 1).any(axis=-1)]
+    cutflow['triggers'] = len(a)
+
+    # At least 2 AK15 jets
+    a = a[ak.count(a['JetsAK15.fCoordinates.fPt'], axis=-1) >= 2]
+    cutflow['n_ak15jets>=2'] = len(a)
+
+    # At least 2 AK4 jets --> deadcells study
+    a = a[ak.count(a['Jets.fCoordinates.fPt'], axis=-1) >= 2]
+    cutflow['n_ak4jets>=2'] = len(a)
+
+    # subleading eta < 2.4 eta
+    a = a[a['JetsAK15.fCoordinates.fEta'][:,1]<2.4]
+    cutflow['ak15j2_eta<2.4'] = len(a)
+
+    # leading and subleading ak4 eta < 2.4 --> deadcells study
+    a = a[a['Jets.fCoordinates.fEta'][:,0]<2.4]
+    a = a[a['Jets.fCoordinates.fEta'][:,1]<2.4]
+    cutflow['ak4j1j2_eta<2.4'] = len(a)
+
+    # lepton vetoes
+    a = a[(a['NMuons']==0) & (a['NElectrons']==0)]
+    cutflow['nleptons=0'] = len(a)
+
+    # MET filters
+    for b in [
+        'HBHENoiseFilter',
+        'HBHEIsoNoiseFilter',
+        'eeBadScFilter',
+        'ecalBadCalibFilter' if UL else 'ecalBadCalibReducedFilter',
+        'BadPFMuonFilter',
+        'BadChargedCandidateFilter',
+        'globalSuperTightHalo2016Filter',
+        ]:
+        a = a[a[b]!=0] # Pass events if not 0, is that correct?
+    cutflow['metfilter'] = len(a)
+    cutflow['preselection'] = len(a)
+
+    copy.array = a
+    logger.debug('cutflow:\n%s', pprint.pformat(copy.cutflow))
+    return copy
+
+
 def filter_preselection(array):
     copy = array.copy()
     a = copy.array
@@ -369,13 +457,13 @@ def filter_at_least_n_jets(array, n=1, cone=8):
     if cone in [8, 15]:
         cone = 'AK' + str(cone)
     elif cone == 4:
-        pass
+        cone = ''
     else:
         raise Exception('Parameter cone should 4, 8, or 15')
     njets = ak.count(array.array['Jets{}.fCoordinates.fPt'.format(cone)], axis=-1)
     copy = array.copy()
     copy.array = copy.array[njets>=n]
-    copy.cutflow['>={}{}jets'.format(n, ak)] = len(copy.array)
+    copy.cutflow['>={}{}jets'.format(n, cone)] = len(copy.array)
     return copy
 
 
@@ -459,13 +547,14 @@ class Columns:
         missing_features = []
         for f in features:
             if f not in self.arrays.keys():
-                logger.error(f'Feature {f} is not available in {self}.')
+                logger.error('Feature %s is not available in %s.', f, self)
                 missing_features.append(f)
         if missing_features:
             raise Exception(
-                f'Cannot build numpy array.'
-                f' Available features: {list(self.arrays.keys())};'
-                f' Missing requested features: {missing_features}'
+                'Cannot build numpy array.'
+                ' Available features: %s;'
+                ' Missing requested features: %s'
+                % list(self.arrays.keys()), missing_features
                 )
         X = []
         for f in features:
@@ -546,9 +635,9 @@ def bdt_feature_columns(array):
     a['ecfd2b1'] = arr['JetsAK15_ecfD2b1'][:,1].to_numpy()
     a['ecfc2b1'] = arr['JetsAK15_ecfC2b1'][:,1].to_numpy()
     a['ecfn2b2'] = arr['JetsAK15_ecfN2b2'][:,1].to_numpy()
-    a['metdphi'] = calc_dphi(arr['JetsAK15.fCoordinates.fEta'][:,1].to_numpy(), arr['METPhi'].to_numpy())
+    a['metdphi'] = calc_dphi(arr['JetsAK15.fCoordinates.fPhi'][:,1].to_numpy(), arr['METPhi'].to_numpy())
 
-    a['weight'] = arr['Weight'].to_numpy()
+    a['weight'] = arr['Weight'].to_numpy() if 'Weight' in arr else np.ones(len(arr))
     a['met'] = arr['MET'].to_numpy()
     a['metphi'] = arr['METPhi'].to_numpy()
 
@@ -568,9 +657,22 @@ def bdt_feature_columns(array):
     a['leading_phi'] = arr['JetsAK15.fCoordinates.fPhi'][:,0].to_numpy()
     a['leading_e'] = arr['JetsAK15.fCoordinates.fE'][:,0].to_numpy()
 
+    a['ak4_lead_eta'] = arr['Jets.fCoordinates.fEta'][:,0].to_numpy()
+    a['ak4_lead_phi'] = arr['Jets.fCoordinates.fPhi'][:,0].to_numpy()
+    a['ak4_lead_pt'] = arr['Jets.fCoordinates.fPt'][:,0].to_numpy()
+    a['ak4_subl_eta'] = arr['Jets.fCoordinates.fEta'][:,1].to_numpy()
+    a['ak4_subl_phi'] = arr['Jets.fCoordinates.fPhi'][:,1].to_numpy()
+    a['ak4_subl_pt'] = arr['Jets.fCoordinates.fPt'][:,1].to_numpy()
+    a['ak8_lead_pt'] = arr['JetsAK8.fCoordinates.fPt'][:,0].to_numpy()
+    a['EcalDeadCellTriggerPrimitiveFilter'] = arr['EcalDeadCellTriggerPrimitiveFilter'].to_numpy()
+    a['EcalDeadCellBoundaryEnergyFilter'] = arr['EcalDeadCellBoundaryEnergyFilter'].to_numpy()
+
     cols.arrays = a
     return cols
 
+# Backward compatibility
+cr_feature_columns = bdt_feature_columns
+
 
 def triggerstudy_columns(array):
     a = array.array # Just to avoid typing array.array everywhere
@@ -634,7 +736,6 @@ def triggerstudy_columns(array):
 def metadata_from_filename(path):
     """
     Heuristic to extract physics parameters from a path and stores it in a metadata dict.
-    
     """
     metadata = {}
     # SIGNAL
@@ -671,13 +772,13 @@ def metadata_from_filename(path):
         left = int(match.group(1))
         right = np.inf if match.group(2) == 'Inf' else int(match.group(2))
         metadata['ht'] = (left, right)
-        logger.debug(f'Setting ht=({metadata["ht"]})')
+        logger.debug('Setting ht=({metadata["ht"]})')
     match = re.search(r'Pt_(\d+)to([\dInf]+)', path)
     if match:
         left = int(match.group(1))
         right = np.inf if match.group(2) == 'Inf' else int(match.group(2))
         metadata['pt'] = (left, right)
-        logger.debug(f'Setting pt=({metadata["pt"]})')
+        logger.debug('Setting pt=({metadata["pt"]})')
     match = re.search(r'genMET\-(\d+)', path)
     if match:
         metadata['genmet'] = int(match.group(1))
